@@ -18,7 +18,7 @@ use RDF::Query::Stream;
 our ($VERSION, $debug);
 BEGIN {
 	$debug		= 0;
-	$VERSION	= do { my @REV = split(/\./, (qw$Revision: 1.12 $)[1]); sprintf("%0.3f", $REV[0] + ($REV[1]/1000)) };
+	$VERSION	= do { my $REV = (qw$Revision: 121 $)[1]; sprintf("%0.3f", 1 + ($REV/1000)) };
 }
 
 ######################################################################
@@ -148,14 +148,17 @@ sub blank_identifier {
 sub add_uri {
 	my $self	= shift;
 	my $uri		= shift;
+	my $named	= shift;
+	
+	my $model	= $self->{model};
 	my $parser	= RDF::Redland::Parser->new();
 	my $redlanduri	= RDF::Redland::URI->new( $uri );
 	my $redlandns	= RDF::Redland::URI->new( $self->base_ns );
-	$parser->parse_into_model(
+	my $stream		= $parser->parse_as_stream(
+		$redlanduri,
 		$redlanduri,
-		$redlandns,
-		$self->{'model'}
 	);
+	$model->add_statements( $stream, ($named ? $redlanduri : ()) );
 }
 
 sub statement_method_map {
@@ -164,50 +167,119 @@ sub statement_method_map {
 
 sub get_statements {
 	my $self	= shift;
-	my @triple	= @_;
+	my @triple	= splice(@_, 0, 3);
+	my $context	= shift;
+	
 	my @defs	= grep defined, @triple;
 	my $model	= $self->{'model'};
 	my $stmt	= RDF::Redland::Statement->new( @triple );
 	my $stream;
 	
-	if (scalar(@defs) == 2) {
-		my @imethods	= qw(sources_iterator arcs_iterator targets_iterator);
-		my @smethods	= qw(subject predicate object);
-		my ($imethod, $smethod);
-		foreach my $i (0 .. 2) {
-			if (not defined $triple[ $i ]) {
-				$imethod	= $imethods[ $i ];
-				$smethod	= $smethods[ $i ];
-				last;
-			}
-		}
-		my $iter	= $model->$imethod( @defs );
+	my %args	= ( bridge => $self, named => 1 );
+	
+	if ($context) {
+		my $iter	= $model->find_statements( $stmt, $context );
+		$args{ context }	= $context;
 		$stream	= sub {
-			return undef unless ($iter);
-			if ($iter->end) {
+			if (@_ and $_[0] eq 'context') {
+				return $context;
+			} elsif (not $iter) {
+				return undef;
+			} elsif ($iter->end) {
 				$iter	= undef;
 				return undef;
+			} else {
+				my $ret	= $iter->current;
+				$iter->next;
+				return $ret;
 			}
-			my $ret	= $iter->current;
-			$iter->next;
-			my $s	= $stmt->clone;
-			$s->$smethod( $ret );
-			return $s;
 		};
 	} else {
-		my $iter	= $model->find_statements( $stmt );
-		$stream	= sub {
-			return undef unless ($iter);
-			if ($iter->end) {
-				$iter	= undef;
-				return undef;
+		if (0) {
+			my $stream	= $model->as_stream();
+			warn "------------------------------\n";
+			while (my $st = $stream->current) {
+				warn $st->as_string;
+			} continue { $stream->next }
+			warn "------------------------------\n";
+		}
+		if (scalar(@defs) == 2) {
+			my @imethods	= qw(sources_iterator arcs_iterator targets_iterator);
+			my @smethods	= qw(subject predicate object);
+			my ($imethod, $smethod);
+			foreach my $i (0 .. 2) {
+				if (not defined $triple[ $i ]) {
+					$imethod	= $imethods[ $i ];
+					$smethod	= $smethods[ $i ];
+					last;
+				}
 			}
-			my $ret	= $iter->current;
-			$iter->next;
-			return $ret;
-		};
+			my $iter	= $model->$imethod( @defs );
+			my $context;
+			$stream	= sub {
+				if (@_ and $_[0] eq 'context') {
+					return $context;
+				} elsif (not $iter) {
+					return undef;
+				} elsif ($iter->end) {
+					$iter	= undef;
+					return undef;
+				} else {
+					my $ret	= $iter->current;
+					$context	= $iter->context;
+					$iter->next;
+					my $s	= $stmt->clone;
+					$s->$smethod( $ret );
+					return $s;
+				}
+			};
+		} else {
+			my $iter	= $model->find_statements( $stmt );
+			warn "iterator: $iter (" . $stmt->as_string . ')' if (0);
+			my $context;
+			$stream	= sub {
+				no warnings 'uninitialized';
+				if (@_ and $_[0] eq 'context') {
+					return $context;
+				} elsif (not $iter) {
+					return undef;
+				} elsif ($iter->end) {
+					$context	= $iter->context;
+					$iter	= undef;
+					return undef;
+				} else {
+					my $ret	= $iter->current;
+					$context	= $iter->context;
+					$iter->next;
+					return $ret;
+				}
+			};
+		}
 	}
-	return RDF::Query::Stream->new( $stream, 'graph', undef, bridge => $self );
+	
+	return RDF::Query::Stream->new( $stream, 'graph', undef, %args );
+}
+
+sub get_context {
+	my $self	= shift;
+	my $stream	= shift;
+	my %args	= @_;
+	
+	if (0) {
+		Carp::cluck "get_context stream: ";
+		local($RDF::Query::debug)	= 2;
+		RDF::Query::_debug_closure( $stream );
+	}
+	
+	my $context	= $stream->('context');
+	return $context;
+}
+
+sub supports {
+	my $self	= shift;
+	my $feature	= shift;
+	return 1 if ($feature eq 'named_graph');
+	return 0;
 }
 
 sub as_xml {
@@ -216,9 +288,10 @@ sub as_xml {
 	return undef unless $iter->is_graph;
 	my $storage	= RDF::Redland::Storage->new("hashes", "test", "new='yes',hash-type='memory'");
 	my $model	= RDF::Redland::Model->new($storage, "");
-	while ($iter and not $iter->finished) {
-		$model->add_statement( $iter->current );
-	} continue { $iter->next }
+	while (my $st = $iter->current) {
+		$model->add_statement( $st );
+		$iter->next;
+	}
 	return $model->to_string;
 }
 
